#include <math.h>
#include "addr_hash.h"

static short int hash_ang(double radian)
{
	return floor(radian * 1000.0);
}

grbs_addr_key_t grbs_addr_new_to_key(const grbs_addr_t *addr)
{
	grbs_addr_key_t res = {0};
	grbs_arc_t *arc;

	switch(addr->type & 0x0f) {
		case ADDR_ARC_CONVEX:  res.is_concave = 0; break;
		case ADDR_ARC_CONCAVE: res.is_concave = 1; break;
		case ADDR_POINT:
			res.pt_uid = addr->obj.pt->uid;
			return res;
		default:
			assert(!"grbs_addr_new_to_key: invalid address type");
			abort();
	}

	/* convex or concave arc */
	arc = addr->obj.arc;
	res.is_ccw = arc->new_adir < 0;
	res.ang = hash_ang((addr->type & ADDR_ARC_END) ? arc->new_sa + arc->new_da : arc->new_sa);
	res.pt_uid = arc->parent_pt->uid;

	for(res.orbit = 0; arc != NULL; arc = arc->link_point.prev)
		res.orbit++;

	return res;
}

grbs_addr_key_t grbs_det_addr_to_key(const grbs_detached_addr_t *det)
{
	grbs_addr_key_t res = {0};
	grbs_arc_t *arc;

	res.pt_uid = det->pt->uid;

	if ((det->type & 0x0f) == ADDR_POINT) return res;

	/* convex or concave arc */
	res.ang = hash_ang((det->type & ADDR_ARC_END) ? det->new_sa + det->new_da : det->new_sa);
	res.is_concave = det->concave;
	res.is_ccw = det->new_adir < 0;

	if (det->arc != NULL) {
		for(res.orbit = 0, arc = det->arc; arc != NULL; arc = arc->link_point.prev)
			if (arc->in_use)
				res.orbit++;
	}
	else
		res.orbit = 1; /* (det->arc == NULL) means first orbit over a new sentinel */

	return res;
}

grbs_addr_key_t grbs_point_to_key(const grbs_point_t *pt)
{
	grbs_addr_key_t res = {0};
	res.pt_uid = pt->uid;
	return res;
}


int grbs_addr_hash_keyeq(grbs_addr_key_t a, grbs_addr_key_t b)
{
	int diff;

	/* cheap tests first */
	if ((a.orbit != b.orbit) || (a.is_concave != b.is_concave) || (a.pt_uid != b.pt_uid) || (a.is_ccw != b.is_ccw))
		return 0;

	/* for angle, allow +-1 for rounding errors */
	diff = a.ang - b.ang;
	return (diff >= -1) && (diff <= 1);
}

